/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
 /* $Id: callbacks.c,v 1.6 2002/10/04 11:36:19 dolphy Exp $ */
 
#include "getcontrol.h"

/* Callbacks */

gint
getcontrol_quit_callback (GtkWidget * widget, gpointer data)
{
	free_images_list ();
	gtk_main_quit ();
}


gint
getcontrol_about_callback (GtkWidget * widget, gpointer data)
{
	GdkPixbuf *pixbuf = NULL;
	static GtkWidget *about = NULL;
	const gchar *authors[] = {
		"Julien MOUTTE. <julien@moutte.net>",
		"Matthias SAOU. <matthias.saou@est.une.marmotte.net>",
		NULL
	};

	/* Documenters credits */

	gchar *documenters[] = {
		NULL
	};

	/* Translator credits */

	gchar *translator_credits = _("translator_credits");
	
	if (about) {
		gtk_window_present (GTK_WINDOW (about));
		return;
	}

	{
		char *filename = NULL;
		filename = gnome_program_locate_file (
				NULL,
				GNOME_FILE_DOMAIN_APP_PIXMAP,
				("getcontrol.png"),
				TRUE, NULL);

		if (filename != NULL)
		{
			pixbuf = gdk_pixbuf_new_from_file(filename, NULL);
			g_free (filename);
		}
	}

	about = gnome_about_new (
			_("Getcontrol"), VERSION,
			"(C) 2002 the Free Software Foundation",
			_("The front end you need for multimedia boxes."),
			(const char **)authors,
			(const char **)documenters,
			strcmp (translator_credits, "translator_credits") != 0 ? translator_credits : NULL,
			pixbuf
				);

	gtk_window_set_transient_for (GTK_WINDOW (about), GTK_WINDOW (app));
	gtk_widget_show (about);
	g_signal_connect (
			G_OBJECT (about), "destroy",
			G_CALLBACK (gtk_widget_destroyed), &about);

	return TRUE;
}

/* Expose callback for the drawing area */
gint
expose_callback (GtkWidget * widget, GdkEventExpose * event, gpointer data)
{
	guchar *pixels;
	int rowstride;

	rowstride = gdk_pixbuf_get_rowstride (frame);

	pixels = gdk_pixbuf_get_pixels (frame) + rowstride * event->area.y + event->area.x * 3;

	gdk_draw_rgb_image_dithalign (
			widget->window,
			widget->style->black_gc,
			event->area.x, event->area.y,
			event->area.width, event->area.height,
			GDK_RGB_DITHER_NORMAL,
			pixels, rowstride,
			event->area.x, event->area.y);

	return TRUE;
}

/* Callback for key_press_event management */
gint
key_pressed_callback (GtkWidget * widget, GdkEventKey * event, gpointer data)
{
	if (!draw_timeout_id) {	
		
		switch (current_status) {
			case IDLE :		
				switch (event->keyval) {
					case 65363 :
						direction = RIGHT;
						break;;
					case 65361 :
						direction = LEFT;
						break;;
					default :
						return TRUE;;
				}
				if (select_timeout_id) { /* User want to rotate more */
			
					gtk_timeout_remove (select_timeout_id);
			
					select_timeout_id = 0;
				}
		
				menu_rotation_manager (NULL);
				break;;
			case SELECTED : /* Key pressed while selected checking if we validate or
							not and disabling screen saver */
				if (screen_saver_timeout_id) {
					gtk_timeout_remove (screen_saver_timeout_id);
					screen_saver_timeout_id = 0;
				}
				switch (event->keyval) {
					case 65363 :
						direction = RIGHT;
						menu_deselection_manager (NULL);
						break;;
					case 65361 :
						direction = LEFT;
						menu_deselection_manager (NULL);
						break;;
					default :
						current_status = ACTIVATING;
						break;;
				}
				break;;
			case SCREEN_SAVING :
				
				if (ss_draw_timeout_id) {
					gtk_timeout_remove (ss_draw_timeout_id);
					ss_draw_timeout_id = 0;
				}
				
				draw_menu();
				
				select_timeout_id = gtk_timeout_add (SELECT_MENU_DELAY, menu_selection_manager, NULL);
				
				break;;
		}
	}
	
	return TRUE;
}

/* Callback for resizing and first drawing */
gint
configure_callback (GtkWidget * widget, gpointer data)
{
	/* Drawing */
	
	draw_menu ();
	
	select_timeout_id = gtk_timeout_add (SELECT_MENU_DELAY, menu_selection_manager, NULL);
	
	return TRUE;
}


