/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
 /* $Id: draw.c,v 1.1 2002/10/03 13:47:08 dolphy Exp $ */
 
#include "getcontrol.h"

/* Our global images list */

image_node * images_list;

/* Draw images

	Function that runs in the images table and draw visible ones
	to their stored coordinates with stored alpha level and zoom
	ratio.
	
*/

gint
draw_images (void)
{
	image_node * draw_images_list = NULL;

	/* Copying the head pointer of the images list */
	
	draw_images_list = images_list;
	
	if (!draw_images_list) /* Images list not initialized */
		return FALSE;
	
	while (draw_images_list) {
		
		GdkRectangle r1, r2, dest;
		
		/*g_message ("image %d infos : angle %d xpos %d ypos %d alpha %d zoom %d visible %d",
			i, images_coord[i][0], images_coord[i][1], images_coord[i][2],
			images_coord[i][3], images_coord[i][4], images_coord[i][5]);*/
		
		if (draw_images_list->visible) { /* If image is visible */
			
			r1.x = draw_images_list->xpos;
			r1.y = draw_images_list->ypos;
			r1.width = draw_images_list->width * draw_images_list->zoom_ratio;
			r1.height = draw_images_list->height * draw_images_list->zoom_ratio;
			
			r2.x = 0;
			r2.y = 0;
			r2.width = back_width;
			r2.height = back_height;
		
			if (gdk_rectangle_intersect (&r1, &r2, &dest)) {
			
				gdk_pixbuf_composite (
					draw_images_list->image,
					frame,
					dest.x, dest.y,
					dest.width, dest.height,
					draw_images_list->xpos, draw_images_list->ypos,
					draw_images_list->zoom_ratio, draw_images_list->zoom_ratio,
					GDK_INTERP_BILINEAR,
					draw_images_list->alpha_level);
				
			}
		}
		
		draw_images_list = draw_images_list->next_image;
		
	}
	
	gtk_widget_draw (da, NULL);

	return TRUE;
}

gint
add_image_at_the_end (image_node * new_image)
{
	image_node * draw_images_list = NULL;

	/* Copying the head pointer of the images list */
	
	draw_images_list = images_list;
	
	if (!draw_images_list) { /* Images list empty */
		images_list = new_image;
		return TRUE;
	}
	
	while (draw_images_list->next_image)
		draw_images_list = draw_images_list->next_image;
	
	draw_images_list->next_image = new_image;
	
	return TRUE;
}

gint
dump_images_list (void)
{
	image_node * draw_images_list = NULL;
	gint i = 0;

	/* Copying the head pointer of the images list */
	
	draw_images_list = images_list;
	
	if (!draw_images_list) { /* Images list not initialized */
		g_message ("no images to dump");
		return FALSE;
	}
	
	while (draw_images_list->next_image) {
		g_message ("image number %d : ", i);
		g_message ("image angle %d : ", draw_images_list->angle);
		g_message ("image xpos %d : ", draw_images_list->xpos);
		g_message ("image ypos %d : ", draw_images_list->ypos);
		g_message ("image width %d : ", draw_images_list->width);
		g_message ("image height %d : ", draw_images_list->height);
		g_message ("image alpha_level %d : ", draw_images_list->alpha_level);
		g_message ("image zoom_ratio %d : ", draw_images_list->zoom_ratio);
		g_message ("image visible %d : ", draw_images_list->visible);
		g_message ("image selected %d : ", draw_images_list->selected);
		draw_images_list = draw_images_list->next_image;
		i++;
	}
	
	return TRUE;
}

gint
free_images_list (void)
{
	image_node * draw_images_list = NULL, * backup = NULL;

	/* Copying the head pointer of the images list */
	
	draw_images_list = images_list;
	
	if (!draw_images_list) { /* Images list not initialized */
		g_message ("no images to free");
		return FALSE;
	}
	
	while (draw_images_list->next_image) {
		
		g_object_unref (draw_images_list->image);
		
		backup = draw_images_list;
		
		draw_images_list = draw_images_list->next_image;
		
		if (backup)
			free (backup);
		
		backup=NULL;
	}
	
	return TRUE;
}
