/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
/* $Id: interface.c,v 1.4 2002/10/04 10:50:31 dolphy Exp $ */

#include "getcontrol.h"

/* Prefs */

GConfClient *conf_client = NULL;

/* Widgets */

GtkWidget *app, *appbar, *da;

/* Current frame */

GdkPixbuf *frame, *frame_backup;

/* Menus */

GnomeUIInfo getcontrolmenu[] = {
	GNOMEUIINFO_MENU_QUIT_ITEM(getcontrol_quit_callback, NULL),
	GNOMEUIINFO_END
};

GnomeUIInfo settingsmenu[] = {
	GNOMEUIINFO_MENU_PREFERENCES_ITEM(getcontrol_about_callback, NULL),
	GNOMEUIINFO_END
};

GnomeUIInfo helpmenu[] = {
	GNOMEUIINFO_HELP("getcontrol"),
	GNOMEUIINFO_MENU_ABOUT_ITEM(getcontrol_about_callback, NULL),
	GNOMEUIINFO_END
};

GnomeUIInfo mainmenu[] = {
	GNOMEUIINFO_MENU_FILE_TREE(getcontrolmenu),
	GNOMEUIINFO_MENU_SETTINGS_TREE(settingsmenu),
	GNOMEUIINFO_MENU_HELP_TREE(helpmenu),
	GNOMEUIINFO_END
};

/* Main */

int
main (int argc, char **argv)
{
	
	/* Defining additional params from command line */
	
	static const struct poptOption options[] = {
		{ NULL, '\0', 0, NULL, 0 }
	}; 

	GnomeClient *client;

	/* Gettext management */

#ifdef ENABLE_NLS
	bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
	textdomain (GETTEXT_PACKAGE);
#endif
	
	/* Initing */
	
	gdk_rgb_init ();
	gnome_program_init (
				"Getcontrol", VERSION,
				LIBGNOMEUI_MODULE,
				argc, argv,
				GNOME_PARAM_POPT_TABLE, options,
				GNOME_PARAM_APP_DATADIR, DATADIR, NULL);

	/* Loading pixbufs */
	
	if (!load_pixbufs ()) {
		g_message ("main(): Could not load all the pixbufs!");
		exit (EXIT_FAILURE);
	}

	client= gnome_master_client ();

	/* Setting icon */
	
	gnome_window_icon_set_default_from_file (GNOME_ICONDIR"/getcontrol.png");

	/* Get the default GConfClient */
	
	conf_client = gconf_client_get_default ();

	/* Creating app */
	
	app = gnome_app_new("getcontrol", _("Getcontrol"));

	gtk_widget_set_usize (GTK_WINDOW(app), back_width, back_height);
	gtk_window_set_resizable (GTK_WINDOW(app), TRUE);

	appbar = gnome_appbar_new(FALSE, TRUE, GNOME_PREFERENCES_USER);
	gnome_app_set_statusbar(GNOME_APP (app), GTK_WIDGET(appbar));

	gnome_appbar_set_status(GNOME_APPBAR (appbar), _("Getcontrol is now running"));

	gnome_app_create_menus(GNOME_APP(app), mainmenu);

	gnome_app_install_menu_hints(GNOME_APP (app), mainmenu);

	/* Connecting signals */
	
	g_signal_connect (
			G_OBJECT (app), "delete_event",
			G_CALLBACK(getcontrol_quit_callback), NULL);

	/* Creating our frames */

	frame = gdk_pixbuf_new (GDK_COLORSPACE_RGB, FALSE, 8, back_width, back_height);
	frame_backup = gdk_pixbuf_new (GDK_COLORSPACE_RGB, FALSE, 8, back_width, back_height);
	
	gtk_widget_push_colormap (gdk_rgb_get_colormap ());

	/* Creating our drawing area */

	da = gtk_drawing_area_new ();
	
	GTK_WIDGET_SET_FLAGS (da, GTK_CAN_FOCUS);
	
	gtk_widget_pop_colormap ();

	/* Connecting expose and button_press_event */

	g_signal_connect (
					GTK_OBJECT (da), "expose_event",
					(GtkSignalFunc) expose_callback, NULL);

	g_signal_connect (
					GTK_OBJECT (da), "key_press_event",
					(GtkSignalFunc) key_pressed_callback, NULL);
					
	g_signal_connect (
					GTK_OBJECT (da), "configure_event",
					(GtkSignalFunc) configure_callback, NULL);

	/* Setting events we would like to receive */

	gtk_widget_set_events (da,
				GDK_EXPOSURE_MASK
				| GDK_LEAVE_NOTIFY_MASK
				| GDK_BUTTON_PRESS_MASK
				| GDK_KEY_PRESS_MASK
				| GDK_KEY_RELEASE_MASK
				| GDK_POINTER_MOTION_MASK
				| GDK_POINTER_MOTION_HINT_MASK);
			 
	gtk_widget_grab_focus ( GTK_WIDGET( da) );
	
	/* Putting our widget in the Gnome app */
	
	gnome_app_set_contents (GNOME_APP (app), da);

	/* Showing widgets */

	gtk_widget_show_all (app);

	/* Starting GTK loop */

	gtk_main ();

	return 0;
}
