/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
 /* $Id: menu.c,v 1.6 2002/10/04 11:36:20 dolphy Exp $ */
 
#include "getcontrol.h"

/* Images list */

const char *image_names[] = {
	"pixmaps/apple-red.png",
	"pixmaps/gnome-applets.png",
	"pixmaps/gnome-calendar.png",
	"pixmaps/gnome-foot.png",
	"pixmaps/gnome-gmush.png",
	"pixmaps/gnome-gimp.png",
	"pixmaps/gnome-gsame.png",
	"pixmaps/gnu-keys.png"
};

gboolean direction = LEFT;

/* Background image and its dimensions */

GdkPixbuf *background;

gint back_width, back_height, xmid, ymid, radius;

/* Angle in degree between each image */

gint ang_step;

/* Number of degrees remaining to rotate */

gint left_to_move, total_move;

/* Global info on what we are doing */

gint current_status = IDLE;

/* Global timer id */

guint select_timeout_id = 0, screen_saver_timeout_id = 0;

/* Animation timers */

guint ss_draw_timeout_id = 0, draw_timeout_id = 0;

/* Loads the images and returns whether the operation succeeded */

gboolean
load_pixbufs (void)
{
	int i, image_coord = 0;
	GError * error = NULL;
	image_node * new_image;

	/* Loading the JPEG background image */

	background = gdk_pixbuf_new_from_file (BACKGROUND_NAME, &error);
	
	if (!background)
		return FALSE;

	/* Getting background size */

	back_width = gdk_pixbuf_get_width (background);
	back_height = gdk_pixbuf_get_height (background);

	/* Middle of the picture */
	
	xmid = back_width / 2.0;
	ymid = back_height / 2.0;

	/* Calculating radius */

	radius = MIN (xmid, ymid) / 2.0;
	
	/* Loading the pictures from table */

	ang_step = 360 / N_IMAGES;
	
	for (i = 0; i < N_IMAGES; i++) {
		
		new_image = (image_node *) malloc (sizeof(image_node));
		
		new_image->image = gdk_pixbuf_new_from_file (image_names[i], &error);
		
		if (!new_image->image)
			return FALSE;
		
		if (!i) {
			image_coord = 0; /* First picture at 0 degree */
		}
		else {
			image_coord = image_coord + ang_step; /* Changing image_coord */
		}
		
		new_image->xpos = 0;
		
		new_image->ypos = 0;
		
		new_image->width = gdk_pixbuf_get_width (new_image->image);
		
		new_image->height = gdk_pixbuf_get_height (new_image->image);
		
		new_image->alpha_level = 255; /* Alpha level to maximum */
		
		new_image->zoom_ratio = new_image->init_zoom_ratio = 1; /* Zoom ratio full size */
		
		new_image->visible = TRUE; /* Visible by default */
		
		new_image->selected = FALSE; /* Is image selected */
		
		new_image->angle = image_coord + ANG_OFFSET; /* Initial menu position */
				
		if (new_image->angle > 360)
			new_image->angle = new_image->angle - 360; /* Staying between 0-360 */
		
		new_image->next_image = NULL;
		
		add_image_at_the_end (new_image);
		
	}

	return TRUE;

}

/* Draw main menu */

gint
draw_menu (void)
{
	image_node * draw_images_list = NULL;

	/* Copying the head pointer of the images list */
	
	draw_images_list = images_list;
	
	if (!draw_images_list) /* Images list not initialized */
		return TRUE;
	
	/* Copying the background picture in frame */
	
	gdk_pixbuf_copy_area (
			background, 0, 0,
			back_width, back_height,
			frame, 0, 0);


	while (draw_images_list) {	

		/* Calculating x, y positions for each picture */
		
		draw_images_list->xpos = xmid + ( radius * cos ( 2.0 * M_PI * draw_images_list->angle / 360) )
								- (draw_images_list->width *  draw_images_list->init_zoom_ratio / 2.0);
		draw_images_list->ypos = ymid - ( radius * sin ( 2.0 * M_PI * draw_images_list->angle / 360) )
								- (draw_images_list->height *  draw_images_list->init_zoom_ratio / 2.0);
		
		draw_images_list->zoom_ratio = draw_images_list->init_zoom_ratio;
		
		draw_images_list->alpha_level = 255;
		
		draw_images_list->visible = TRUE; /* Visible */
		
		draw_images_list = draw_images_list->next_image;
	}
	
	/* Now drawing */
	
	draw_images ();
	
	current_status = IDLE;
	
	return TRUE;
}

/* Taking care of rotating the main menu */

gint
menu_rotation_manager (gpointer data)
{
	gint rotate, alpha_level;
	gdouble zoom_ratio;
	image_node * draw_images_list = NULL;

	/* Copying the head pointer of the images list */
	
	draw_images_list = images_list;
	
	if ( (!draw_images_list) || !(draw_images_list->next_image) ) /* Images list not initialized */
		return FALSE;
	
	if ( !draw_timeout_id ) { /* No timer installed */
		
		/* Set the lock while rotating */
		
		current_status = ROTATING;
		
		/* Setting the number of degrees to move */
		
		left_to_move = total_move = ang_step;
		
		/* Telling what we are doing */
		
		if (direction == LEFT)		
			gnome_appbar_set_status(GNOME_APPBAR (appbar), _("Rotating <-"));
		else 
			gnome_appbar_set_status(GNOME_APPBAR (appbar), _("Rotating ->"));
		
		/* Installing timer for animation */
		
		draw_timeout_id = gtk_timeout_add (MAIN_MENU_FRAME_DELAY, menu_rotation_manager, NULL);
	
	}
		
	if (!left_to_move) { /* We finnished our move */
		
		gtk_timeout_remove ( draw_timeout_id); /* Disabling timer */
		
		draw_timeout_id = 0; /* Resetting draw timer */
		
		select_timeout_id = gtk_timeout_add (SELECT_MENU_DELAY, menu_selection_manager, NULL);
		
		gnome_appbar_set_status(GNOME_APPBAR (appbar), _("Done rotation")); /* Telling status */
		
		current_status = IDLE;
		
		return TRUE;
	}
	
	if (direction == LEFT) {
		if (left_to_move <= ROT_OFFSET) {
			alpha_level = 255;
			zoom_ratio = 1; /* On the last move we finnish on original zoom_ratio and alpha */
			rotate = left_to_move; /* Needed rotation is smaller than the preferred one */
		}
		else {
			alpha_level = 255 - ( ( ( cos ( ( ( left_to_move * M_PI * 2.0 ) / total_move ) + M_PI ) + 1 ) / 2 ) * ALPHA_THRESOLD );
			
			/* Here we calculate a zoom_ratio modificator which will be used to impact the initial zoom
			ratio. This way the initial zoom ratio can change the modificator will keep a coherent
			aspect. This ratio will vary from 1 to 2 following cosinus graph from Pi on a 2 Pi period */
			
			zoom_ratio = ( cos ( ( ( left_to_move * M_PI * 2.0 ) / total_move ) + M_PI ) + 1 ) / 2.0 + 1;
			
			rotate = ROT_OFFSET;
		}
	}
	else {
		if (left_to_move <= ROT_OFFSET) {
			alpha_level = 255;
			zoom_ratio = 1; /* On the last move we finnish on original zoom_ratio and alpha */
			rotate = 0 - left_to_move; /* Needed rotation is smaller than the preferred one */
		}
		else {
			alpha_level = 255 - ( ( ( cos ( ( ( left_to_move * M_PI * 2.0 ) / total_move ) + M_PI ) + 1 ) / 2 ) * ALPHA_THRESOLD );
			
			/* Here we calculate a zoom_ratio modificator which will be used to impact the initial zoom
			ratio. This way the initial zoom ratio can change the modificator will keep a coherent
			aspect. This ratio will vary from 1 to 2 following cosinus graph from Pi on a 2 Pi period */
			
			zoom_ratio = ( cos ( ( ( left_to_move * M_PI * 2.0 ) / total_move ) + M_PI ) + 1 ) / 2.0 + 1;
			rotate = 0 - ROT_OFFSET;
		}
	}

	while (draw_images_list) {
		
		draw_images_list->angle = draw_images_list->angle + rotate; /* Changing image angle */
		
		if (draw_images_list->angle >= 360) /* We stay between 0-360 */
			draw_images_list->angle = draw_images_list->angle - 360;
		if (draw_images_list->angle <= 0) /* We stay between 0-360 */
			draw_images_list->angle = 360 + draw_images_list->angle;

		draw_images_list->zoom_ratio = draw_images_list->init_zoom_ratio / zoom_ratio;
		
		draw_images_list->xpos = xmid + ( radius * cos ( 2.0 * M_PI * draw_images_list->angle / 360) )
								- ( draw_images_list->width * draw_images_list->zoom_ratio / 2.0);
		draw_images_list->ypos = ymid - ( radius * sin ( 2.0 * M_PI * draw_images_list->angle / 360) )
								- (draw_images_list->height * draw_images_list->zoom_ratio / 2.0);
			
		draw_images_list->alpha_level = alpha_level;
		
		draw_images_list->visible = TRUE; /* Visible */
		
		draw_images_list = draw_images_list->next_image;
	}
	
	if (direction == LEFT)
		left_to_move = left_to_move - rotate; /* Removing the rotation we've done */
	else
		left_to_move = left_to_move - (0 -rotate); /* Removing the rotation we've done */
	
	
	/* Copying the background picture in frame */
	
	gdk_pixbuf_copy_area (
			background, 0, 0,
			back_width, back_height,
			frame, 0, 0);
	
	draw_images ();
	
	return TRUE;
}

/* Taking care of animating menu selection */

gint
menu_selection_manager (gpointer data)
{
	gint i, move, alpha_level;
	gdouble zoom_ratio;
	image_node * draw_images_list = NULL;

	/* Copying the head pointer of the images list */
	
	draw_images_list = images_list;
	
	if (!draw_images_list) /* Images list not initialized */
		return FALSE;
	
	if ( !draw_timeout_id ) { /* No timer installed */
		
		gtk_timeout_remove ( select_timeout_id); /* Disabling selection timer */
		
		current_status = SELECTING;
		
		/* Drawing all pictures except selected menu */
		
		while (draw_images_list) {
			
			if ( !(draw_images_list->angle - ANG_OFFSET) ) {

				draw_images_list->selected = TRUE;
				draw_images_list->visible = FALSE;
				
				/* How many pixel from img's center to go to central place
	
				left_to_move = total_move = ymid - ( draw_images_list->ypos
					+ ( draw_images_list->height *  draw_images_list->init_zoom_ratio / 2.0 ) );*/
				
				left_to_move = total_move = radius;
	
			}
			else {
				draw_images_list->selected = FALSE;
				draw_images_list->visible = TRUE;
			}
			
			draw_images_list = draw_images_list->next_image;
		}
		
		/* Copying the background picture in frame */
	
		gdk_pixbuf_copy_area (
			background, 0, 0,
			back_width, back_height,
			frame, 0, 0);
		
		/* Now drawing images */
		
		draw_images ();
			
		/* Copying our drawing area in a backup frame */
	
		gdk_pixbuf_copy_area (
			frame, 0, 0,
			back_width, back_height,
			frame_backup, 0, 0);
		
		/* Installing timer for animation */
		
		draw_timeout_id = gtk_timeout_add (SELECTED_MENU_FRAME_DELAY, menu_selection_manager, NULL);
		
		return TRUE;

	}
		
	if (!left_to_move) { /* We finnished our move */
		
		gtk_timeout_remove ( draw_timeout_id); /* Disabling timer */
		
		draw_timeout_id = 0; /* Resetting draw timer */
		
		current_status = SELECTED;
		
		if (!screen_saver_timeout_id) /* Installing timer for screen saver */
					screen_saver_timeout_id = gtk_timeout_add (SCREEN_SAVER_DELAY, screen_saver_manager, NULL);
		
		return TRUE;
	}
	
	if (left_to_move <= MOVE_OFFSET) {
		move = left_to_move; /* Needed rotation is smaller than the preferred one */
		alpha_level = 255;
		zoom_ratio = ( radius * 2.0 - (draw_images_list->height * draw_images_list->init_zoom_ratio) - BORDER_THRESOLD)
			/ draw_images_list->height;
	}
	else {
		move = MOVE_OFFSET;
		alpha_level = 255 - ( ( ( cos ( ( ( left_to_move * M_PI * 2.0 ) / total_move ) + M_PI ) + 1 ) / 2 ) * ALPHA_THRESOLD );
		zoom_ratio = cos ( ( ( left_to_move * ( M_PI / 2.0 ) ) / total_move ) + ( M_PI * 2.0 / 3.0 ) ) + 1.5;
	}
	
	while (draw_images_list) {
			
			if (draw_images_list->selected) {
				draw_images_list->ypos = ( draw_images_list->ypos + ( draw_images_list->height *  draw_images_list->zoom_ratio / 2.0 )
					+ move ) - ( draw_images_list->height * zoom_ratio / 2.0 );
				draw_images_list->zoom_ratio = zoom_ratio;
				draw_images_list->xpos = xmid - ( draw_images_list->width *  draw_images_list->zoom_ratio / 2.0 );
				draw_images_list->alpha_level = alpha_level;
				draw_images_list->visible = TRUE;
			}
			else
				draw_images_list->visible = FALSE;
			
			draw_images_list = draw_images_list->next_image;
	}	

	left_to_move = left_to_move - move; /* Removing the move we've done */
	
	/* Copying the background picture in frame */
	
	gdk_pixbuf_copy_area (
			frame_backup, 0, 0,
			back_width, back_height,
			frame, 0, 0);
	
	draw_images ();
	
	return TRUE;
}

/* Taking care of animating menu deselection */

gint
menu_deselection_manager (gpointer data)
{
	gint i, move, alpha_level;
	gdouble zoom_ratio;
	image_node * draw_images_list = NULL;

	/* Copying the head pointer of the images list */
	
	draw_images_list = images_list;
	
	if (!draw_images_list) /* Images list not initialized */
		return FALSE;
	
	if ( !draw_timeout_id ) { /* No timer installed */
		
		gtk_timeout_remove ( select_timeout_id); /* Disabling selection timer */
		
		current_status = DESELECTING;
		
		/* Drawing all pictures except selected menu */
		
		while (draw_images_list) {
			
			if ( !(draw_images_list->angle - ANG_OFFSET) ) {

				draw_images_list->selected = TRUE;
				draw_images_list->visible = FALSE;
				
				/* How many pixel from img's center to go to central place */
	
				left_to_move = total_move = radius;
	
			}
			else {
				draw_images_list->selected = FALSE;
				draw_images_list->visible = TRUE;
			}
			
			draw_images_list = draw_images_list->next_image;
		}
		
		/* Copying the background picture in frame */
	
		gdk_pixbuf_copy_area (
			background, 0, 0,
			back_width, back_height,
			frame, 0, 0);
		
		/* Now drawing images */
		
		draw_images ();
			
		/* Copying our drawing area in a backup frame */
	
		gdk_pixbuf_copy_area (
			frame, 0, 0,
			back_width, back_height,
			frame_backup, 0, 0);
		
		/* Installing timer for animation */
		
		draw_timeout_id = gtk_timeout_add (SELECTED_MENU_FRAME_DELAY, menu_deselection_manager, NULL);
		
		return TRUE;

	}
		
	if (!left_to_move) { /* We finnished our move */
		
		gtk_timeout_remove ( draw_timeout_id); /* Disabling timer */
		
		draw_timeout_id = 0; /* Resetting draw timer */
		
		current_status = IDLE;
		
		menu_rotation_manager (NULL);
		
		return TRUE;
	}
	
	if (left_to_move <= MOVE_OFFSET) {
		move = left_to_move; /* Needed rotation is smaller than the preferred one */
		alpha_level = 255;
		zoom_ratio = 1;
	}
	else {
		move = MOVE_OFFSET;
		alpha_level = 255 - ( ( ( cos ( ( ( left_to_move * M_PI * 2.0 ) / total_move ) + M_PI ) + 1 ) / 2 ) * ALPHA_THRESOLD );
		zoom_ratio = cos ( ( ( left_to_move * ( M_PI / 2.0 ) ) / total_move ) + ( M_PI * 2.0 / 3.0 ) ) + 1.5;
	}
	
	while (draw_images_list) {
			
			if (draw_images_list->selected) {
				draw_images_list->ypos = ( draw_images_list->ypos + ( draw_images_list->height *  draw_images_list->zoom_ratio / 2.0 )
					- move ) - ( draw_images_list->height * zoom_ratio / 2.0 );
				draw_images_list->zoom_ratio = zoom_ratio;
				draw_images_list->xpos = xmid - ( draw_images_list->width *  draw_images_list->zoom_ratio / 2.0 );
				draw_images_list->alpha_level = alpha_level;
				draw_images_list->visible = TRUE;
			}
			else
				draw_images_list->visible = FALSE;
			
			draw_images_list = draw_images_list->next_image;
	}	

	left_to_move = left_to_move - move; /* Removing the move we've done */
	
	/* Copying the background picture in frame */
	
	gdk_pixbuf_copy_area (
			frame_backup, 0, 0,
			back_width, back_height,
			frame, 0, 0);
	
	draw_images ();
	
	return TRUE;
}

/* Screensaver manager */

gint frame_num;

gint
screen_saver_manager (gpointer data)
{
	double f;
	int i=0;
	image_node * draw_images_list = NULL;
	
	/* Copying the head pointer of the images list */
	
	draw_images_list = images_list;
	
	if (screen_saver_timeout_id) {
		gtk_timeout_remove (screen_saver_timeout_id);
		screen_saver_timeout_id = 0;
	}
	
	if (!ss_draw_timeout_id) {
		current_status = SCREEN_SAVING;
		
		/* Installing timer for animation */
		
		ss_draw_timeout_id = gtk_timeout_add (SCREEN_SAVER_FRAME_DELAY, screen_saver_manager, NULL);
	}
		
	gdk_pixbuf_copy_area (background, 0, 0, back_width, back_height,
			      frame, 0, 0);

	f = (double) (frame_num % 60) / 60;

	while (draw_images_list) {
		gdouble ang;
		gdouble r;
		gdouble k;

		ang = 2.0 * M_PI * (gdouble) i / N_IMAGES - f * 2.0 * M_PI;

		r = radius + (radius / 3.0) * sin (f * 2.0 * M_PI);

		draw_images_list->xpos = floor (xmid + r * cos (ang) - draw_images_list->width / 2.0 + 0.5);
		draw_images_list->ypos = floor (ymid + r * sin (ang) - draw_images_list->height / 2.0 + 0.5);

		k = (i & 1) ? sin (f * 2.0 * M_PI) : cos (f * 2.0 * M_PI);
		k = 2.0 * k * k;
		k = MAX (0.25, k);

		draw_images_list->zoom_ratio = k;
		
		draw_images_list->alpha_level = ((i & 1)
					       ? MAX (127, fabs (255 * sin (f * 2.0 * M_PI)))
					       : MAX (127, fabs (255 * cos (f * 2.0 * M_PI))));

		draw_images_list->visible = TRUE;
		draw_images_list->selected = FALSE;
		
		draw_images_list = draw_images_list->next_image;
		
		i++;
	}

	//dump_images_list ();
	
	draw_images ();
	
	frame_num++;
	return TRUE;
}
